/* -*- Mode: C; indent-tabs-mode: t; tab-width: 4 -*-
// ---------------------------------------------------------------------------
// SquirrelJME
//     Copyright (C) Stephanie Gawroriski <xer@multiphasicapps.net>
// ---------------------------------------------------------------------------
// SquirrelJME is under the Mozilla Public License Version 2.0.
// See license.mkd for licensing and copyright information.
// -------------------------------------------------------------------------*/

#include "lib/scritchui/scritchui.h"
#include "lib/scritchui/scritchuiPencil.h"
#include "lib/scritchui/scritchuiTypes.h"
#include "sjme/debug.h"

#define SJME_BITLINE_SET_USE(id) \
	SJME_TOKEN_PASTE_PP(sjme_scritchui_bitLine, id)

#define SJME_BITLINE_BEGIN(id) \
	static sjme_errorCode SJME_BITLINE_SET_USE(id) \
	(sjme_scritchui_pencil g, \
	sjme_jint x, sjme_jint y)

#define SJME_BITLINE_DEF \
	sjme_errorCode error = SJME_ERROR_NONE; \
	sjme_scritchui_pencilDrawPixelFunc pixel; \
	sjme_scritchui_pencilDrawHorizFunc horiz; \
	 \
	pixel = g->prim.drawPixel; \
	horiz = g->prim.drawHoriz

#define SJME_BITLINE_RET return error

#define SJME_BITPIXL(d) \
	error |= pixel(g, x + d, y)

#define SJME_BITLINE(d, l) \
	error |= horiz(g, x + d, y, l)

/** Bitline 0 (00000000). */
SJME_BITLINE_BEGIN(0)
{
	SJME_BITLINE_DEF;
	
	
	SJME_BITLINE_RET;
}

/** Bitline 1 (00000001). */
SJME_BITLINE_BEGIN(1)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	
	SJME_BITLINE_RET;
}

/** Bitline 2 (00000010). */
SJME_BITLINE_BEGIN(2)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(1);
	
	SJME_BITLINE_RET;
}

/** Bitline 3 (00000011). */
SJME_BITLINE_BEGIN(3)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 4 (00000100). */
SJME_BITLINE_BEGIN(4)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(2);
	
	SJME_BITLINE_RET;
}

/** Bitline 5 (00000101). */
SJME_BITLINE_BEGIN(5)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITPIXL(2);
	
	SJME_BITLINE_RET;
}

/** Bitline 6 (00000110). */
SJME_BITLINE_BEGIN(6)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(1, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 7 (00000111). */
SJME_BITLINE_BEGIN(7)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 3);
	
	SJME_BITLINE_RET;
}

/** Bitline 8 (00001000). */
SJME_BITLINE_BEGIN(8)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(3);
	
	SJME_BITLINE_RET;
}

/** Bitline 9 (00001001). */
SJME_BITLINE_BEGIN(9)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITPIXL(3);
	
	SJME_BITLINE_RET;
}

/** Bitline 10 (00001010). */
SJME_BITLINE_BEGIN(10)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(1);
	SJME_BITPIXL(3);
	
	SJME_BITLINE_RET;
}

/** Bitline 11 (00001011). */
SJME_BITLINE_BEGIN(11)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 2);
	SJME_BITPIXL(3);
	
	SJME_BITLINE_RET;
}

/** Bitline 12 (00001100). */
SJME_BITLINE_BEGIN(12)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(2, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 13 (00001101). */
SJME_BITLINE_BEGIN(13)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITLINE(2, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 14 (00001110). */
SJME_BITLINE_BEGIN(14)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(1, 3);
	
	SJME_BITLINE_RET;
}

/** Bitline 15 (00001111). */
SJME_BITLINE_BEGIN(15)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 4);
	
	SJME_BITLINE_RET;
}

/** Bitline 16 (00010000). */
SJME_BITLINE_BEGIN(16)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(4);
	
	SJME_BITLINE_RET;
}

/** Bitline 17 (00010001). */
SJME_BITLINE_BEGIN(17)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITPIXL(4);
	
	SJME_BITLINE_RET;
}

/** Bitline 18 (00010010). */
SJME_BITLINE_BEGIN(18)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(1);
	SJME_BITPIXL(4);
	
	SJME_BITLINE_RET;
}

/** Bitline 19 (00010011). */
SJME_BITLINE_BEGIN(19)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 2);
	SJME_BITPIXL(4);
	
	SJME_BITLINE_RET;
}

/** Bitline 20 (00010100). */
SJME_BITLINE_BEGIN(20)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(2);
	SJME_BITPIXL(4);
	
	SJME_BITLINE_RET;
}

/** Bitline 21 (00010101). */
SJME_BITLINE_BEGIN(21)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITPIXL(2);
	SJME_BITPIXL(4);
	
	SJME_BITLINE_RET;
}

/** Bitline 22 (00010110). */
SJME_BITLINE_BEGIN(22)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(1, 2);
	SJME_BITPIXL(4);
	
	SJME_BITLINE_RET;
}

/** Bitline 23 (00010111). */
SJME_BITLINE_BEGIN(23)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 3);
	SJME_BITPIXL(4);
	
	SJME_BITLINE_RET;
}

/** Bitline 24 (00011000). */
SJME_BITLINE_BEGIN(24)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(3, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 25 (00011001). */
SJME_BITLINE_BEGIN(25)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITLINE(3, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 26 (00011010). */
SJME_BITLINE_BEGIN(26)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(1);
	SJME_BITLINE(3, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 27 (00011011). */
SJME_BITLINE_BEGIN(27)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 2);
	SJME_BITLINE(3, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 28 (00011100). */
SJME_BITLINE_BEGIN(28)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(2, 3);
	
	SJME_BITLINE_RET;
}

/** Bitline 29 (00011101). */
SJME_BITLINE_BEGIN(29)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITLINE(2, 3);
	
	SJME_BITLINE_RET;
}

/** Bitline 30 (00011110). */
SJME_BITLINE_BEGIN(30)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(1, 4);
	
	SJME_BITLINE_RET;
}

/** Bitline 31 (00011111). */
SJME_BITLINE_BEGIN(31)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 5);
	
	SJME_BITLINE_RET;
}

/** Bitline 32 (00100000). */
SJME_BITLINE_BEGIN(32)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(5);
	
	SJME_BITLINE_RET;
}

/** Bitline 33 (00100001). */
SJME_BITLINE_BEGIN(33)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITPIXL(5);
	
	SJME_BITLINE_RET;
}

/** Bitline 34 (00100010). */
SJME_BITLINE_BEGIN(34)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(1);
	SJME_BITPIXL(5);
	
	SJME_BITLINE_RET;
}

/** Bitline 35 (00100011). */
SJME_BITLINE_BEGIN(35)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 2);
	SJME_BITPIXL(5);
	
	SJME_BITLINE_RET;
}

/** Bitline 36 (00100100). */
SJME_BITLINE_BEGIN(36)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(2);
	SJME_BITPIXL(5);
	
	SJME_BITLINE_RET;
}

/** Bitline 37 (00100101). */
SJME_BITLINE_BEGIN(37)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITPIXL(2);
	SJME_BITPIXL(5);
	
	SJME_BITLINE_RET;
}

/** Bitline 38 (00100110). */
SJME_BITLINE_BEGIN(38)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(1, 2);
	SJME_BITPIXL(5);
	
	SJME_BITLINE_RET;
}

/** Bitline 39 (00100111). */
SJME_BITLINE_BEGIN(39)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 3);
	SJME_BITPIXL(5);
	
	SJME_BITLINE_RET;
}

/** Bitline 40 (00101000). */
SJME_BITLINE_BEGIN(40)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(3);
	SJME_BITPIXL(5);
	
	SJME_BITLINE_RET;
}

/** Bitline 41 (00101001). */
SJME_BITLINE_BEGIN(41)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITPIXL(3);
	SJME_BITPIXL(5);
	
	SJME_BITLINE_RET;
}

/** Bitline 42 (00101010). */
SJME_BITLINE_BEGIN(42)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(1);
	SJME_BITPIXL(3);
	SJME_BITPIXL(5);
	
	SJME_BITLINE_RET;
}

/** Bitline 43 (00101011). */
SJME_BITLINE_BEGIN(43)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 2);
	SJME_BITPIXL(3);
	SJME_BITPIXL(5);
	
	SJME_BITLINE_RET;
}

/** Bitline 44 (00101100). */
SJME_BITLINE_BEGIN(44)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(2, 2);
	SJME_BITPIXL(5);
	
	SJME_BITLINE_RET;
}

/** Bitline 45 (00101101). */
SJME_BITLINE_BEGIN(45)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITLINE(2, 2);
	SJME_BITPIXL(5);
	
	SJME_BITLINE_RET;
}

/** Bitline 46 (00101110). */
SJME_BITLINE_BEGIN(46)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(1, 3);
	SJME_BITPIXL(5);
	
	SJME_BITLINE_RET;
}

/** Bitline 47 (00101111). */
SJME_BITLINE_BEGIN(47)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 4);
	SJME_BITPIXL(5);
	
	SJME_BITLINE_RET;
}

/** Bitline 48 (00110000). */
SJME_BITLINE_BEGIN(48)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(4, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 49 (00110001). */
SJME_BITLINE_BEGIN(49)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITLINE(4, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 50 (00110010). */
SJME_BITLINE_BEGIN(50)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(1);
	SJME_BITLINE(4, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 51 (00110011). */
SJME_BITLINE_BEGIN(51)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 2);
	SJME_BITLINE(4, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 52 (00110100). */
SJME_BITLINE_BEGIN(52)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(2);
	SJME_BITLINE(4, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 53 (00110101). */
SJME_BITLINE_BEGIN(53)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITPIXL(2);
	SJME_BITLINE(4, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 54 (00110110). */
SJME_BITLINE_BEGIN(54)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(1, 2);
	SJME_BITLINE(4, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 55 (00110111). */
SJME_BITLINE_BEGIN(55)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 3);
	SJME_BITLINE(4, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 56 (00111000). */
SJME_BITLINE_BEGIN(56)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(3, 3);
	
	SJME_BITLINE_RET;
}

/** Bitline 57 (00111001). */
SJME_BITLINE_BEGIN(57)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITLINE(3, 3);
	
	SJME_BITLINE_RET;
}

/** Bitline 58 (00111010). */
SJME_BITLINE_BEGIN(58)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(1);
	SJME_BITLINE(3, 3);
	
	SJME_BITLINE_RET;
}

/** Bitline 59 (00111011). */
SJME_BITLINE_BEGIN(59)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 2);
	SJME_BITLINE(3, 3);
	
	SJME_BITLINE_RET;
}

/** Bitline 60 (00111100). */
SJME_BITLINE_BEGIN(60)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(2, 4);
	
	SJME_BITLINE_RET;
}

/** Bitline 61 (00111101). */
SJME_BITLINE_BEGIN(61)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITLINE(2, 4);
	
	SJME_BITLINE_RET;
}

/** Bitline 62 (00111110). */
SJME_BITLINE_BEGIN(62)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(1, 5);
	
	SJME_BITLINE_RET;
}

/** Bitline 63 (00111111). */
SJME_BITLINE_BEGIN(63)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 6);
	
	SJME_BITLINE_RET;
}

/** Bitline 64 (01000000). */
SJME_BITLINE_BEGIN(64)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(6);
	
	SJME_BITLINE_RET;
}

/** Bitline 65 (01000001). */
SJME_BITLINE_BEGIN(65)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITPIXL(6);
	
	SJME_BITLINE_RET;
}

/** Bitline 66 (01000010). */
SJME_BITLINE_BEGIN(66)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(1);
	SJME_BITPIXL(6);
	
	SJME_BITLINE_RET;
}

/** Bitline 67 (01000011). */
SJME_BITLINE_BEGIN(67)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 2);
	SJME_BITPIXL(6);
	
	SJME_BITLINE_RET;
}

/** Bitline 68 (01000100). */
SJME_BITLINE_BEGIN(68)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(2);
	SJME_BITPIXL(6);
	
	SJME_BITLINE_RET;
}

/** Bitline 69 (01000101). */
SJME_BITLINE_BEGIN(69)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITPIXL(2);
	SJME_BITPIXL(6);
	
	SJME_BITLINE_RET;
}

/** Bitline 70 (01000110). */
SJME_BITLINE_BEGIN(70)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(1, 2);
	SJME_BITPIXL(6);
	
	SJME_BITLINE_RET;
}

/** Bitline 71 (01000111). */
SJME_BITLINE_BEGIN(71)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 3);
	SJME_BITPIXL(6);
	
	SJME_BITLINE_RET;
}

/** Bitline 72 (01001000). */
SJME_BITLINE_BEGIN(72)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(3);
	SJME_BITPIXL(6);
	
	SJME_BITLINE_RET;
}

/** Bitline 73 (01001001). */
SJME_BITLINE_BEGIN(73)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITPIXL(3);
	SJME_BITPIXL(6);
	
	SJME_BITLINE_RET;
}

/** Bitline 74 (01001010). */
SJME_BITLINE_BEGIN(74)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(1);
	SJME_BITPIXL(3);
	SJME_BITPIXL(6);
	
	SJME_BITLINE_RET;
}

/** Bitline 75 (01001011). */
SJME_BITLINE_BEGIN(75)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 2);
	SJME_BITPIXL(3);
	SJME_BITPIXL(6);
	
	SJME_BITLINE_RET;
}

/** Bitline 76 (01001100). */
SJME_BITLINE_BEGIN(76)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(2, 2);
	SJME_BITPIXL(6);
	
	SJME_BITLINE_RET;
}

/** Bitline 77 (01001101). */
SJME_BITLINE_BEGIN(77)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITLINE(2, 2);
	SJME_BITPIXL(6);
	
	SJME_BITLINE_RET;
}

/** Bitline 78 (01001110). */
SJME_BITLINE_BEGIN(78)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(1, 3);
	SJME_BITPIXL(6);
	
	SJME_BITLINE_RET;
}

/** Bitline 79 (01001111). */
SJME_BITLINE_BEGIN(79)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 4);
	SJME_BITPIXL(6);
	
	SJME_BITLINE_RET;
}

/** Bitline 80 (01010000). */
SJME_BITLINE_BEGIN(80)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(4);
	SJME_BITPIXL(6);
	
	SJME_BITLINE_RET;
}

/** Bitline 81 (01010001). */
SJME_BITLINE_BEGIN(81)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITPIXL(4);
	SJME_BITPIXL(6);
	
	SJME_BITLINE_RET;
}

/** Bitline 82 (01010010). */
SJME_BITLINE_BEGIN(82)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(1);
	SJME_BITPIXL(4);
	SJME_BITPIXL(6);
	
	SJME_BITLINE_RET;
}

/** Bitline 83 (01010011). */
SJME_BITLINE_BEGIN(83)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 2);
	SJME_BITPIXL(4);
	SJME_BITPIXL(6);
	
	SJME_BITLINE_RET;
}

/** Bitline 84 (01010100). */
SJME_BITLINE_BEGIN(84)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(2);
	SJME_BITPIXL(4);
	SJME_BITPIXL(6);
	
	SJME_BITLINE_RET;
}

/** Bitline 85 (01010101). */
SJME_BITLINE_BEGIN(85)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITPIXL(2);
	SJME_BITPIXL(4);
	SJME_BITPIXL(6);
	
	SJME_BITLINE_RET;
}

/** Bitline 86 (01010110). */
SJME_BITLINE_BEGIN(86)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(1, 2);
	SJME_BITPIXL(4);
	SJME_BITPIXL(6);
	
	SJME_BITLINE_RET;
}

/** Bitline 87 (01010111). */
SJME_BITLINE_BEGIN(87)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 3);
	SJME_BITPIXL(4);
	SJME_BITPIXL(6);
	
	SJME_BITLINE_RET;
}

/** Bitline 88 (01011000). */
SJME_BITLINE_BEGIN(88)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(3, 2);
	SJME_BITPIXL(6);
	
	SJME_BITLINE_RET;
}

/** Bitline 89 (01011001). */
SJME_BITLINE_BEGIN(89)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITLINE(3, 2);
	SJME_BITPIXL(6);
	
	SJME_BITLINE_RET;
}

/** Bitline 90 (01011010). */
SJME_BITLINE_BEGIN(90)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(1);
	SJME_BITLINE(3, 2);
	SJME_BITPIXL(6);
	
	SJME_BITLINE_RET;
}

/** Bitline 91 (01011011). */
SJME_BITLINE_BEGIN(91)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 2);
	SJME_BITLINE(3, 2);
	SJME_BITPIXL(6);
	
	SJME_BITLINE_RET;
}

/** Bitline 92 (01011100). */
SJME_BITLINE_BEGIN(92)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(2, 3);
	SJME_BITPIXL(6);
	
	SJME_BITLINE_RET;
}

/** Bitline 93 (01011101). */
SJME_BITLINE_BEGIN(93)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITLINE(2, 3);
	SJME_BITPIXL(6);
	
	SJME_BITLINE_RET;
}

/** Bitline 94 (01011110). */
SJME_BITLINE_BEGIN(94)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(1, 4);
	SJME_BITPIXL(6);
	
	SJME_BITLINE_RET;
}

/** Bitline 95 (01011111). */
SJME_BITLINE_BEGIN(95)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 5);
	SJME_BITPIXL(6);
	
	SJME_BITLINE_RET;
}

/** Bitline 96 (01100000). */
SJME_BITLINE_BEGIN(96)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(5, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 97 (01100001). */
SJME_BITLINE_BEGIN(97)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITLINE(5, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 98 (01100010). */
SJME_BITLINE_BEGIN(98)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(1);
	SJME_BITLINE(5, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 99 (01100011). */
SJME_BITLINE_BEGIN(99)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 2);
	SJME_BITLINE(5, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 100 (01100100). */
SJME_BITLINE_BEGIN(100)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(2);
	SJME_BITLINE(5, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 101 (01100101). */
SJME_BITLINE_BEGIN(101)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITPIXL(2);
	SJME_BITLINE(5, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 102 (01100110). */
SJME_BITLINE_BEGIN(102)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(1, 2);
	SJME_BITLINE(5, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 103 (01100111). */
SJME_BITLINE_BEGIN(103)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 3);
	SJME_BITLINE(5, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 104 (01101000). */
SJME_BITLINE_BEGIN(104)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(3);
	SJME_BITLINE(5, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 105 (01101001). */
SJME_BITLINE_BEGIN(105)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITPIXL(3);
	SJME_BITLINE(5, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 106 (01101010). */
SJME_BITLINE_BEGIN(106)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(1);
	SJME_BITPIXL(3);
	SJME_BITLINE(5, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 107 (01101011). */
SJME_BITLINE_BEGIN(107)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 2);
	SJME_BITPIXL(3);
	SJME_BITLINE(5, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 108 (01101100). */
SJME_BITLINE_BEGIN(108)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(2, 2);
	SJME_BITLINE(5, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 109 (01101101). */
SJME_BITLINE_BEGIN(109)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITLINE(2, 2);
	SJME_BITLINE(5, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 110 (01101110). */
SJME_BITLINE_BEGIN(110)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(1, 3);
	SJME_BITLINE(5, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 111 (01101111). */
SJME_BITLINE_BEGIN(111)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 4);
	SJME_BITLINE(5, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 112 (01110000). */
SJME_BITLINE_BEGIN(112)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(4, 3);
	
	SJME_BITLINE_RET;
}

/** Bitline 113 (01110001). */
SJME_BITLINE_BEGIN(113)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITLINE(4, 3);
	
	SJME_BITLINE_RET;
}

/** Bitline 114 (01110010). */
SJME_BITLINE_BEGIN(114)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(1);
	SJME_BITLINE(4, 3);
	
	SJME_BITLINE_RET;
}

/** Bitline 115 (01110011). */
SJME_BITLINE_BEGIN(115)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 2);
	SJME_BITLINE(4, 3);
	
	SJME_BITLINE_RET;
}

/** Bitline 116 (01110100). */
SJME_BITLINE_BEGIN(116)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(2);
	SJME_BITLINE(4, 3);
	
	SJME_BITLINE_RET;
}

/** Bitline 117 (01110101). */
SJME_BITLINE_BEGIN(117)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITPIXL(2);
	SJME_BITLINE(4, 3);
	
	SJME_BITLINE_RET;
}

/** Bitline 118 (01110110). */
SJME_BITLINE_BEGIN(118)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(1, 2);
	SJME_BITLINE(4, 3);
	
	SJME_BITLINE_RET;
}

/** Bitline 119 (01110111). */
SJME_BITLINE_BEGIN(119)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 3);
	SJME_BITLINE(4, 3);
	
	SJME_BITLINE_RET;
}

/** Bitline 120 (01111000). */
SJME_BITLINE_BEGIN(120)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(3, 4);
	
	SJME_BITLINE_RET;
}

/** Bitline 121 (01111001). */
SJME_BITLINE_BEGIN(121)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITLINE(3, 4);
	
	SJME_BITLINE_RET;
}

/** Bitline 122 (01111010). */
SJME_BITLINE_BEGIN(122)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(1);
	SJME_BITLINE(3, 4);
	
	SJME_BITLINE_RET;
}

/** Bitline 123 (01111011). */
SJME_BITLINE_BEGIN(123)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 2);
	SJME_BITLINE(3, 4);
	
	SJME_BITLINE_RET;
}

/** Bitline 124 (01111100). */
SJME_BITLINE_BEGIN(124)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(2, 5);
	
	SJME_BITLINE_RET;
}

/** Bitline 125 (01111101). */
SJME_BITLINE_BEGIN(125)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITLINE(2, 5);
	
	SJME_BITLINE_RET;
}

/** Bitline 126 (01111110). */
SJME_BITLINE_BEGIN(126)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(1, 6);
	
	SJME_BITLINE_RET;
}

/** Bitline 127 (01111111). */
SJME_BITLINE_BEGIN(127)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 7);
	
	SJME_BITLINE_RET;
}

/** Bitline 128 (10000000). */
SJME_BITLINE_BEGIN(128)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 129 (10000001). */
SJME_BITLINE_BEGIN(129)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 130 (10000010). */
SJME_BITLINE_BEGIN(130)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(1);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 131 (10000011). */
SJME_BITLINE_BEGIN(131)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 2);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 132 (10000100). */
SJME_BITLINE_BEGIN(132)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(2);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 133 (10000101). */
SJME_BITLINE_BEGIN(133)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITPIXL(2);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 134 (10000110). */
SJME_BITLINE_BEGIN(134)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(1, 2);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 135 (10000111). */
SJME_BITLINE_BEGIN(135)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 3);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 136 (10001000). */
SJME_BITLINE_BEGIN(136)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(3);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 137 (10001001). */
SJME_BITLINE_BEGIN(137)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITPIXL(3);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 138 (10001010). */
SJME_BITLINE_BEGIN(138)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(1);
	SJME_BITPIXL(3);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 139 (10001011). */
SJME_BITLINE_BEGIN(139)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 2);
	SJME_BITPIXL(3);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 140 (10001100). */
SJME_BITLINE_BEGIN(140)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(2, 2);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 141 (10001101). */
SJME_BITLINE_BEGIN(141)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITLINE(2, 2);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 142 (10001110). */
SJME_BITLINE_BEGIN(142)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(1, 3);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 143 (10001111). */
SJME_BITLINE_BEGIN(143)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 4);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 144 (10010000). */
SJME_BITLINE_BEGIN(144)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(4);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 145 (10010001). */
SJME_BITLINE_BEGIN(145)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITPIXL(4);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 146 (10010010). */
SJME_BITLINE_BEGIN(146)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(1);
	SJME_BITPIXL(4);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 147 (10010011). */
SJME_BITLINE_BEGIN(147)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 2);
	SJME_BITPIXL(4);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 148 (10010100). */
SJME_BITLINE_BEGIN(148)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(2);
	SJME_BITPIXL(4);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 149 (10010101). */
SJME_BITLINE_BEGIN(149)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITPIXL(2);
	SJME_BITPIXL(4);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 150 (10010110). */
SJME_BITLINE_BEGIN(150)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(1, 2);
	SJME_BITPIXL(4);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 151 (10010111). */
SJME_BITLINE_BEGIN(151)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 3);
	SJME_BITPIXL(4);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 152 (10011000). */
SJME_BITLINE_BEGIN(152)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(3, 2);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 153 (10011001). */
SJME_BITLINE_BEGIN(153)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITLINE(3, 2);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 154 (10011010). */
SJME_BITLINE_BEGIN(154)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(1);
	SJME_BITLINE(3, 2);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 155 (10011011). */
SJME_BITLINE_BEGIN(155)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 2);
	SJME_BITLINE(3, 2);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 156 (10011100). */
SJME_BITLINE_BEGIN(156)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(2, 3);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 157 (10011101). */
SJME_BITLINE_BEGIN(157)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITLINE(2, 3);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 158 (10011110). */
SJME_BITLINE_BEGIN(158)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(1, 4);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 159 (10011111). */
SJME_BITLINE_BEGIN(159)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 5);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 160 (10100000). */
SJME_BITLINE_BEGIN(160)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(5);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 161 (10100001). */
SJME_BITLINE_BEGIN(161)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITPIXL(5);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 162 (10100010). */
SJME_BITLINE_BEGIN(162)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(1);
	SJME_BITPIXL(5);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 163 (10100011). */
SJME_BITLINE_BEGIN(163)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 2);
	SJME_BITPIXL(5);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 164 (10100100). */
SJME_BITLINE_BEGIN(164)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(2);
	SJME_BITPIXL(5);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 165 (10100101). */
SJME_BITLINE_BEGIN(165)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITPIXL(2);
	SJME_BITPIXL(5);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 166 (10100110). */
SJME_BITLINE_BEGIN(166)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(1, 2);
	SJME_BITPIXL(5);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 167 (10100111). */
SJME_BITLINE_BEGIN(167)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 3);
	SJME_BITPIXL(5);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 168 (10101000). */
SJME_BITLINE_BEGIN(168)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(3);
	SJME_BITPIXL(5);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 169 (10101001). */
SJME_BITLINE_BEGIN(169)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITPIXL(3);
	SJME_BITPIXL(5);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 170 (10101010). */
SJME_BITLINE_BEGIN(170)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(1);
	SJME_BITPIXL(3);
	SJME_BITPIXL(5);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 171 (10101011). */
SJME_BITLINE_BEGIN(171)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 2);
	SJME_BITPIXL(3);
	SJME_BITPIXL(5);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 172 (10101100). */
SJME_BITLINE_BEGIN(172)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(2, 2);
	SJME_BITPIXL(5);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 173 (10101101). */
SJME_BITLINE_BEGIN(173)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITLINE(2, 2);
	SJME_BITPIXL(5);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 174 (10101110). */
SJME_BITLINE_BEGIN(174)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(1, 3);
	SJME_BITPIXL(5);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 175 (10101111). */
SJME_BITLINE_BEGIN(175)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 4);
	SJME_BITPIXL(5);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 176 (10110000). */
SJME_BITLINE_BEGIN(176)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(4, 2);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 177 (10110001). */
SJME_BITLINE_BEGIN(177)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITLINE(4, 2);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 178 (10110010). */
SJME_BITLINE_BEGIN(178)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(1);
	SJME_BITLINE(4, 2);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 179 (10110011). */
SJME_BITLINE_BEGIN(179)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 2);
	SJME_BITLINE(4, 2);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 180 (10110100). */
SJME_BITLINE_BEGIN(180)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(2);
	SJME_BITLINE(4, 2);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 181 (10110101). */
SJME_BITLINE_BEGIN(181)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITPIXL(2);
	SJME_BITLINE(4, 2);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 182 (10110110). */
SJME_BITLINE_BEGIN(182)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(1, 2);
	SJME_BITLINE(4, 2);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 183 (10110111). */
SJME_BITLINE_BEGIN(183)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 3);
	SJME_BITLINE(4, 2);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 184 (10111000). */
SJME_BITLINE_BEGIN(184)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(3, 3);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 185 (10111001). */
SJME_BITLINE_BEGIN(185)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITLINE(3, 3);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 186 (10111010). */
SJME_BITLINE_BEGIN(186)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(1);
	SJME_BITLINE(3, 3);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 187 (10111011). */
SJME_BITLINE_BEGIN(187)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 2);
	SJME_BITLINE(3, 3);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 188 (10111100). */
SJME_BITLINE_BEGIN(188)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(2, 4);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 189 (10111101). */
SJME_BITLINE_BEGIN(189)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITLINE(2, 4);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 190 (10111110). */
SJME_BITLINE_BEGIN(190)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(1, 5);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 191 (10111111). */
SJME_BITLINE_BEGIN(191)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 6);
	SJME_BITPIXL(7);
	
	SJME_BITLINE_RET;
}

/** Bitline 192 (11000000). */
SJME_BITLINE_BEGIN(192)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(6, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 193 (11000001). */
SJME_BITLINE_BEGIN(193)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITLINE(6, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 194 (11000010). */
SJME_BITLINE_BEGIN(194)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(1);
	SJME_BITLINE(6, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 195 (11000011). */
SJME_BITLINE_BEGIN(195)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 2);
	SJME_BITLINE(6, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 196 (11000100). */
SJME_BITLINE_BEGIN(196)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(2);
	SJME_BITLINE(6, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 197 (11000101). */
SJME_BITLINE_BEGIN(197)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITPIXL(2);
	SJME_BITLINE(6, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 198 (11000110). */
SJME_BITLINE_BEGIN(198)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(1, 2);
	SJME_BITLINE(6, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 199 (11000111). */
SJME_BITLINE_BEGIN(199)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 3);
	SJME_BITLINE(6, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 200 (11001000). */
SJME_BITLINE_BEGIN(200)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(3);
	SJME_BITLINE(6, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 201 (11001001). */
SJME_BITLINE_BEGIN(201)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITPIXL(3);
	SJME_BITLINE(6, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 202 (11001010). */
SJME_BITLINE_BEGIN(202)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(1);
	SJME_BITPIXL(3);
	SJME_BITLINE(6, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 203 (11001011). */
SJME_BITLINE_BEGIN(203)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 2);
	SJME_BITPIXL(3);
	SJME_BITLINE(6, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 204 (11001100). */
SJME_BITLINE_BEGIN(204)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(2, 2);
	SJME_BITLINE(6, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 205 (11001101). */
SJME_BITLINE_BEGIN(205)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITLINE(2, 2);
	SJME_BITLINE(6, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 206 (11001110). */
SJME_BITLINE_BEGIN(206)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(1, 3);
	SJME_BITLINE(6, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 207 (11001111). */
SJME_BITLINE_BEGIN(207)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 4);
	SJME_BITLINE(6, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 208 (11010000). */
SJME_BITLINE_BEGIN(208)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(4);
	SJME_BITLINE(6, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 209 (11010001). */
SJME_BITLINE_BEGIN(209)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITPIXL(4);
	SJME_BITLINE(6, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 210 (11010010). */
SJME_BITLINE_BEGIN(210)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(1);
	SJME_BITPIXL(4);
	SJME_BITLINE(6, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 211 (11010011). */
SJME_BITLINE_BEGIN(211)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 2);
	SJME_BITPIXL(4);
	SJME_BITLINE(6, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 212 (11010100). */
SJME_BITLINE_BEGIN(212)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(2);
	SJME_BITPIXL(4);
	SJME_BITLINE(6, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 213 (11010101). */
SJME_BITLINE_BEGIN(213)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITPIXL(2);
	SJME_BITPIXL(4);
	SJME_BITLINE(6, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 214 (11010110). */
SJME_BITLINE_BEGIN(214)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(1, 2);
	SJME_BITPIXL(4);
	SJME_BITLINE(6, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 215 (11010111). */
SJME_BITLINE_BEGIN(215)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 3);
	SJME_BITPIXL(4);
	SJME_BITLINE(6, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 216 (11011000). */
SJME_BITLINE_BEGIN(216)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(3, 2);
	SJME_BITLINE(6, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 217 (11011001). */
SJME_BITLINE_BEGIN(217)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITLINE(3, 2);
	SJME_BITLINE(6, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 218 (11011010). */
SJME_BITLINE_BEGIN(218)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(1);
	SJME_BITLINE(3, 2);
	SJME_BITLINE(6, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 219 (11011011). */
SJME_BITLINE_BEGIN(219)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 2);
	SJME_BITLINE(3, 2);
	SJME_BITLINE(6, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 220 (11011100). */
SJME_BITLINE_BEGIN(220)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(2, 3);
	SJME_BITLINE(6, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 221 (11011101). */
SJME_BITLINE_BEGIN(221)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITLINE(2, 3);
	SJME_BITLINE(6, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 222 (11011110). */
SJME_BITLINE_BEGIN(222)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(1, 4);
	SJME_BITLINE(6, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 223 (11011111). */
SJME_BITLINE_BEGIN(223)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 5);
	SJME_BITLINE(6, 2);
	
	SJME_BITLINE_RET;
}

/** Bitline 224 (11100000). */
SJME_BITLINE_BEGIN(224)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(5, 3);
	
	SJME_BITLINE_RET;
}

/** Bitline 225 (11100001). */
SJME_BITLINE_BEGIN(225)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITLINE(5, 3);
	
	SJME_BITLINE_RET;
}

/** Bitline 226 (11100010). */
SJME_BITLINE_BEGIN(226)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(1);
	SJME_BITLINE(5, 3);
	
	SJME_BITLINE_RET;
}

/** Bitline 227 (11100011). */
SJME_BITLINE_BEGIN(227)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 2);
	SJME_BITLINE(5, 3);
	
	SJME_BITLINE_RET;
}

/** Bitline 228 (11100100). */
SJME_BITLINE_BEGIN(228)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(2);
	SJME_BITLINE(5, 3);
	
	SJME_BITLINE_RET;
}

/** Bitline 229 (11100101). */
SJME_BITLINE_BEGIN(229)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITPIXL(2);
	SJME_BITLINE(5, 3);
	
	SJME_BITLINE_RET;
}

/** Bitline 230 (11100110). */
SJME_BITLINE_BEGIN(230)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(1, 2);
	SJME_BITLINE(5, 3);
	
	SJME_BITLINE_RET;
}

/** Bitline 231 (11100111). */
SJME_BITLINE_BEGIN(231)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 3);
	SJME_BITLINE(5, 3);
	
	SJME_BITLINE_RET;
}

/** Bitline 232 (11101000). */
SJME_BITLINE_BEGIN(232)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(3);
	SJME_BITLINE(5, 3);
	
	SJME_BITLINE_RET;
}

/** Bitline 233 (11101001). */
SJME_BITLINE_BEGIN(233)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITPIXL(3);
	SJME_BITLINE(5, 3);
	
	SJME_BITLINE_RET;
}

/** Bitline 234 (11101010). */
SJME_BITLINE_BEGIN(234)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(1);
	SJME_BITPIXL(3);
	SJME_BITLINE(5, 3);
	
	SJME_BITLINE_RET;
}

/** Bitline 235 (11101011). */
SJME_BITLINE_BEGIN(235)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 2);
	SJME_BITPIXL(3);
	SJME_BITLINE(5, 3);
	
	SJME_BITLINE_RET;
}

/** Bitline 236 (11101100). */
SJME_BITLINE_BEGIN(236)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(2, 2);
	SJME_BITLINE(5, 3);
	
	SJME_BITLINE_RET;
}

/** Bitline 237 (11101101). */
SJME_BITLINE_BEGIN(237)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITLINE(2, 2);
	SJME_BITLINE(5, 3);
	
	SJME_BITLINE_RET;
}

/** Bitline 238 (11101110). */
SJME_BITLINE_BEGIN(238)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(1, 3);
	SJME_BITLINE(5, 3);
	
	SJME_BITLINE_RET;
}

/** Bitline 239 (11101111). */
SJME_BITLINE_BEGIN(239)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 4);
	SJME_BITLINE(5, 3);
	
	SJME_BITLINE_RET;
}

/** Bitline 240 (11110000). */
SJME_BITLINE_BEGIN(240)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(4, 4);
	
	SJME_BITLINE_RET;
}

/** Bitline 241 (11110001). */
SJME_BITLINE_BEGIN(241)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITLINE(4, 4);
	
	SJME_BITLINE_RET;
}

/** Bitline 242 (11110010). */
SJME_BITLINE_BEGIN(242)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(1);
	SJME_BITLINE(4, 4);
	
	SJME_BITLINE_RET;
}

/** Bitline 243 (11110011). */
SJME_BITLINE_BEGIN(243)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 2);
	SJME_BITLINE(4, 4);
	
	SJME_BITLINE_RET;
}

/** Bitline 244 (11110100). */
SJME_BITLINE_BEGIN(244)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(2);
	SJME_BITLINE(4, 4);
	
	SJME_BITLINE_RET;
}

/** Bitline 245 (11110101). */
SJME_BITLINE_BEGIN(245)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITPIXL(2);
	SJME_BITLINE(4, 4);
	
	SJME_BITLINE_RET;
}

/** Bitline 246 (11110110). */
SJME_BITLINE_BEGIN(246)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(1, 2);
	SJME_BITLINE(4, 4);
	
	SJME_BITLINE_RET;
}

/** Bitline 247 (11110111). */
SJME_BITLINE_BEGIN(247)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 3);
	SJME_BITLINE(4, 4);
	
	SJME_BITLINE_RET;
}

/** Bitline 248 (11111000). */
SJME_BITLINE_BEGIN(248)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(3, 5);
	
	SJME_BITLINE_RET;
}

/** Bitline 249 (11111001). */
SJME_BITLINE_BEGIN(249)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITLINE(3, 5);
	
	SJME_BITLINE_RET;
}

/** Bitline 250 (11111010). */
SJME_BITLINE_BEGIN(250)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(1);
	SJME_BITLINE(3, 5);
	
	SJME_BITLINE_RET;
}

/** Bitline 251 (11111011). */
SJME_BITLINE_BEGIN(251)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 2);
	SJME_BITLINE(3, 5);
	
	SJME_BITLINE_RET;
}

/** Bitline 252 (11111100). */
SJME_BITLINE_BEGIN(252)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(2, 6);
	
	SJME_BITLINE_RET;
}

/** Bitline 253 (11111101). */
SJME_BITLINE_BEGIN(253)
{
	SJME_BITLINE_DEF;
	
	SJME_BITPIXL(0);
	SJME_BITLINE(2, 6);
	
	SJME_BITLINE_RET;
}

/** Bitline 254 (11111110). */
SJME_BITLINE_BEGIN(254)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(1, 7);
	
	SJME_BITLINE_RET;
}

/** Bitline 255 (11111111). */
SJME_BITLINE_BEGIN(255)
{
	SJME_BITLINE_DEF;
	
	SJME_BITLINE(0, 8);
	
	SJME_BITLINE_RET;
}

const sjme_scritchui_pencilBitLineFunc sjme_scritchui_pencilBitLines[256] =
{
	SJME_BITLINE_SET_USE(0),
	SJME_BITLINE_SET_USE(1),
	SJME_BITLINE_SET_USE(2),
	SJME_BITLINE_SET_USE(3),
	SJME_BITLINE_SET_USE(4),
	SJME_BITLINE_SET_USE(5),
	SJME_BITLINE_SET_USE(6),
	SJME_BITLINE_SET_USE(7),
	SJME_BITLINE_SET_USE(8),
	SJME_BITLINE_SET_USE(9),
	SJME_BITLINE_SET_USE(10),
	SJME_BITLINE_SET_USE(11),
	SJME_BITLINE_SET_USE(12),
	SJME_BITLINE_SET_USE(13),
	SJME_BITLINE_SET_USE(14),
	SJME_BITLINE_SET_USE(15),
	SJME_BITLINE_SET_USE(16),
	SJME_BITLINE_SET_USE(17),
	SJME_BITLINE_SET_USE(18),
	SJME_BITLINE_SET_USE(19),
	SJME_BITLINE_SET_USE(20),
	SJME_BITLINE_SET_USE(21),
	SJME_BITLINE_SET_USE(22),
	SJME_BITLINE_SET_USE(23),
	SJME_BITLINE_SET_USE(24),
	SJME_BITLINE_SET_USE(25),
	SJME_BITLINE_SET_USE(26),
	SJME_BITLINE_SET_USE(27),
	SJME_BITLINE_SET_USE(28),
	SJME_BITLINE_SET_USE(29),
	SJME_BITLINE_SET_USE(30),
	SJME_BITLINE_SET_USE(31),
	SJME_BITLINE_SET_USE(32),
	SJME_BITLINE_SET_USE(33),
	SJME_BITLINE_SET_USE(34),
	SJME_BITLINE_SET_USE(35),
	SJME_BITLINE_SET_USE(36),
	SJME_BITLINE_SET_USE(37),
	SJME_BITLINE_SET_USE(38),
	SJME_BITLINE_SET_USE(39),
	SJME_BITLINE_SET_USE(40),
	SJME_BITLINE_SET_USE(41),
	SJME_BITLINE_SET_USE(42),
	SJME_BITLINE_SET_USE(43),
	SJME_BITLINE_SET_USE(44),
	SJME_BITLINE_SET_USE(45),
	SJME_BITLINE_SET_USE(46),
	SJME_BITLINE_SET_USE(47),
	SJME_BITLINE_SET_USE(48),
	SJME_BITLINE_SET_USE(49),
	SJME_BITLINE_SET_USE(50),
	SJME_BITLINE_SET_USE(51),
	SJME_BITLINE_SET_USE(52),
	SJME_BITLINE_SET_USE(53),
	SJME_BITLINE_SET_USE(54),
	SJME_BITLINE_SET_USE(55),
	SJME_BITLINE_SET_USE(56),
	SJME_BITLINE_SET_USE(57),
	SJME_BITLINE_SET_USE(58),
	SJME_BITLINE_SET_USE(59),
	SJME_BITLINE_SET_USE(60),
	SJME_BITLINE_SET_USE(61),
	SJME_BITLINE_SET_USE(62),
	SJME_BITLINE_SET_USE(63),
	SJME_BITLINE_SET_USE(64),
	SJME_BITLINE_SET_USE(65),
	SJME_BITLINE_SET_USE(66),
	SJME_BITLINE_SET_USE(67),
	SJME_BITLINE_SET_USE(68),
	SJME_BITLINE_SET_USE(69),
	SJME_BITLINE_SET_USE(70),
	SJME_BITLINE_SET_USE(71),
	SJME_BITLINE_SET_USE(72),
	SJME_BITLINE_SET_USE(73),
	SJME_BITLINE_SET_USE(74),
	SJME_BITLINE_SET_USE(75),
	SJME_BITLINE_SET_USE(76),
	SJME_BITLINE_SET_USE(77),
	SJME_BITLINE_SET_USE(78),
	SJME_BITLINE_SET_USE(79),
	SJME_BITLINE_SET_USE(80),
	SJME_BITLINE_SET_USE(81),
	SJME_BITLINE_SET_USE(82),
	SJME_BITLINE_SET_USE(83),
	SJME_BITLINE_SET_USE(84),
	SJME_BITLINE_SET_USE(85),
	SJME_BITLINE_SET_USE(86),
	SJME_BITLINE_SET_USE(87),
	SJME_BITLINE_SET_USE(88),
	SJME_BITLINE_SET_USE(89),
	SJME_BITLINE_SET_USE(90),
	SJME_BITLINE_SET_USE(91),
	SJME_BITLINE_SET_USE(92),
	SJME_BITLINE_SET_USE(93),
	SJME_BITLINE_SET_USE(94),
	SJME_BITLINE_SET_USE(95),
	SJME_BITLINE_SET_USE(96),
	SJME_BITLINE_SET_USE(97),
	SJME_BITLINE_SET_USE(98),
	SJME_BITLINE_SET_USE(99),
	SJME_BITLINE_SET_USE(100),
	SJME_BITLINE_SET_USE(101),
	SJME_BITLINE_SET_USE(102),
	SJME_BITLINE_SET_USE(103),
	SJME_BITLINE_SET_USE(104),
	SJME_BITLINE_SET_USE(105),
	SJME_BITLINE_SET_USE(106),
	SJME_BITLINE_SET_USE(107),
	SJME_BITLINE_SET_USE(108),
	SJME_BITLINE_SET_USE(109),
	SJME_BITLINE_SET_USE(110),
	SJME_BITLINE_SET_USE(111),
	SJME_BITLINE_SET_USE(112),
	SJME_BITLINE_SET_USE(113),
	SJME_BITLINE_SET_USE(114),
	SJME_BITLINE_SET_USE(115),
	SJME_BITLINE_SET_USE(116),
	SJME_BITLINE_SET_USE(117),
	SJME_BITLINE_SET_USE(118),
	SJME_BITLINE_SET_USE(119),
	SJME_BITLINE_SET_USE(120),
	SJME_BITLINE_SET_USE(121),
	SJME_BITLINE_SET_USE(122),
	SJME_BITLINE_SET_USE(123),
	SJME_BITLINE_SET_USE(124),
	SJME_BITLINE_SET_USE(125),
	SJME_BITLINE_SET_USE(126),
	SJME_BITLINE_SET_USE(127),
	SJME_BITLINE_SET_USE(128),
	SJME_BITLINE_SET_USE(129),
	SJME_BITLINE_SET_USE(130),
	SJME_BITLINE_SET_USE(131),
	SJME_BITLINE_SET_USE(132),
	SJME_BITLINE_SET_USE(133),
	SJME_BITLINE_SET_USE(134),
	SJME_BITLINE_SET_USE(135),
	SJME_BITLINE_SET_USE(136),
	SJME_BITLINE_SET_USE(137),
	SJME_BITLINE_SET_USE(138),
	SJME_BITLINE_SET_USE(139),
	SJME_BITLINE_SET_USE(140),
	SJME_BITLINE_SET_USE(141),
	SJME_BITLINE_SET_USE(142),
	SJME_BITLINE_SET_USE(143),
	SJME_BITLINE_SET_USE(144),
	SJME_BITLINE_SET_USE(145),
	SJME_BITLINE_SET_USE(146),
	SJME_BITLINE_SET_USE(147),
	SJME_BITLINE_SET_USE(148),
	SJME_BITLINE_SET_USE(149),
	SJME_BITLINE_SET_USE(150),
	SJME_BITLINE_SET_USE(151),
	SJME_BITLINE_SET_USE(152),
	SJME_BITLINE_SET_USE(153),
	SJME_BITLINE_SET_USE(154),
	SJME_BITLINE_SET_USE(155),
	SJME_BITLINE_SET_USE(156),
	SJME_BITLINE_SET_USE(157),
	SJME_BITLINE_SET_USE(158),
	SJME_BITLINE_SET_USE(159),
	SJME_BITLINE_SET_USE(160),
	SJME_BITLINE_SET_USE(161),
	SJME_BITLINE_SET_USE(162),
	SJME_BITLINE_SET_USE(163),
	SJME_BITLINE_SET_USE(164),
	SJME_BITLINE_SET_USE(165),
	SJME_BITLINE_SET_USE(166),
	SJME_BITLINE_SET_USE(167),
	SJME_BITLINE_SET_USE(168),
	SJME_BITLINE_SET_USE(169),
	SJME_BITLINE_SET_USE(170),
	SJME_BITLINE_SET_USE(171),
	SJME_BITLINE_SET_USE(172),
	SJME_BITLINE_SET_USE(173),
	SJME_BITLINE_SET_USE(174),
	SJME_BITLINE_SET_USE(175),
	SJME_BITLINE_SET_USE(176),
	SJME_BITLINE_SET_USE(177),
	SJME_BITLINE_SET_USE(178),
	SJME_BITLINE_SET_USE(179),
	SJME_BITLINE_SET_USE(180),
	SJME_BITLINE_SET_USE(181),
	SJME_BITLINE_SET_USE(182),
	SJME_BITLINE_SET_USE(183),
	SJME_BITLINE_SET_USE(184),
	SJME_BITLINE_SET_USE(185),
	SJME_BITLINE_SET_USE(186),
	SJME_BITLINE_SET_USE(187),
	SJME_BITLINE_SET_USE(188),
	SJME_BITLINE_SET_USE(189),
	SJME_BITLINE_SET_USE(190),
	SJME_BITLINE_SET_USE(191),
	SJME_BITLINE_SET_USE(192),
	SJME_BITLINE_SET_USE(193),
	SJME_BITLINE_SET_USE(194),
	SJME_BITLINE_SET_USE(195),
	SJME_BITLINE_SET_USE(196),
	SJME_BITLINE_SET_USE(197),
	SJME_BITLINE_SET_USE(198),
	SJME_BITLINE_SET_USE(199),
	SJME_BITLINE_SET_USE(200),
	SJME_BITLINE_SET_USE(201),
	SJME_BITLINE_SET_USE(202),
	SJME_BITLINE_SET_USE(203),
	SJME_BITLINE_SET_USE(204),
	SJME_BITLINE_SET_USE(205),
	SJME_BITLINE_SET_USE(206),
	SJME_BITLINE_SET_USE(207),
	SJME_BITLINE_SET_USE(208),
	SJME_BITLINE_SET_USE(209),
	SJME_BITLINE_SET_USE(210),
	SJME_BITLINE_SET_USE(211),
	SJME_BITLINE_SET_USE(212),
	SJME_BITLINE_SET_USE(213),
	SJME_BITLINE_SET_USE(214),
	SJME_BITLINE_SET_USE(215),
	SJME_BITLINE_SET_USE(216),
	SJME_BITLINE_SET_USE(217),
	SJME_BITLINE_SET_USE(218),
	SJME_BITLINE_SET_USE(219),
	SJME_BITLINE_SET_USE(220),
	SJME_BITLINE_SET_USE(221),
	SJME_BITLINE_SET_USE(222),
	SJME_BITLINE_SET_USE(223),
	SJME_BITLINE_SET_USE(224),
	SJME_BITLINE_SET_USE(225),
	SJME_BITLINE_SET_USE(226),
	SJME_BITLINE_SET_USE(227),
	SJME_BITLINE_SET_USE(228),
	SJME_BITLINE_SET_USE(229),
	SJME_BITLINE_SET_USE(230),
	SJME_BITLINE_SET_USE(231),
	SJME_BITLINE_SET_USE(232),
	SJME_BITLINE_SET_USE(233),
	SJME_BITLINE_SET_USE(234),
	SJME_BITLINE_SET_USE(235),
	SJME_BITLINE_SET_USE(236),
	SJME_BITLINE_SET_USE(237),
	SJME_BITLINE_SET_USE(238),
	SJME_BITLINE_SET_USE(239),
	SJME_BITLINE_SET_USE(240),
	SJME_BITLINE_SET_USE(241),
	SJME_BITLINE_SET_USE(242),
	SJME_BITLINE_SET_USE(243),
	SJME_BITLINE_SET_USE(244),
	SJME_BITLINE_SET_USE(245),
	SJME_BITLINE_SET_USE(246),
	SJME_BITLINE_SET_USE(247),
	SJME_BITLINE_SET_USE(248),
	SJME_BITLINE_SET_USE(249),
	SJME_BITLINE_SET_USE(250),
	SJME_BITLINE_SET_USE(251),
	SJME_BITLINE_SET_USE(252),
	SJME_BITLINE_SET_USE(253),
	SJME_BITLINE_SET_USE(254),
	SJME_BITLINE_SET_USE(255),
};
